<?php

namespace Service;

use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Permission {
    /**
     * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Permission
     */
    public $permissionRepository;
    protected $entityManager;
    protected $permissionFactory;

    const IS_MANAGER = Entities\Permission::TYPE_MANAGER;
    const IS_RESOURCE = Entities\Permission::TYPE_RESOURCE;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Factory\Permission $permissionFactory){
        $this->entityManager = $entityManager;
        $this->permissionFactory = $permissionFactory;
        $this->permissionRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Permission');
    }

    function createPermission($resource, $structure, $type = self::IS_RESOURCE){
        $permission = $this->permissionFactory->createPermission($resource, $structure, $type);
        $this->entityManager->persist($permission);
        $this->entityManager->flush();
        return $permission;
    }

    function updatePermission($permission, $resource, $structure, $type = self::IS_RESOURCE){
        $permissionModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Permission';
        if(!($permission instanceof $permissionModel)){
            $permission = $this->entityManager->find($permissionModel, $permission);
        }
        $permission = $this->permissionFactory->updatePermission($permission, $resource, $structure, $type);
        $this->entityManager->persist($permission);
        $this->entityManager->flush();
        return $permission;
    }

    function persistPermission($permission = null, $resource, $structure, $type = 'R'){
        $resourceModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource';
        $structureModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure';
        if(!($resource instanceof $resourceModel)){
            $resource = $this->entityManager->find($resourceModel, $resource);
        }
        if(!($structure instanceof $structureModel)){
            $structure = $this->entityManager->find($structureModel, $structure);
        }
        if(empty($permission)){
            return $this->createPermission($resource, $structure, $type);
        }
        else{
            return $this->updatePermission($permission, $resource, $structure, $type);
        }
    }

    function removeNotFoundPermission($structure, $resources, $type){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Permission[] $permissions */
        $permissions = $this->permissionRepository->findBy(array('structure' => $structure,'type'=>$type));
        foreach($permissions as $permission){
            if(!in_array($permission->getResource(), $resources)){
                $this->entityManager->remove($permission);
            }
        }
        $this->entityManager->flush();
    }

} 