<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Zeedhi\DevOps\Model\Devops\Entities as EntitiesDevOps;

/**
 * Class Task
 *
 * @package Service
 */
class TaskWorkitem {

	/** @var EntityManager $entityManager */
	protected $entityManager;
	/** @var \Util\Utilities $environment */
	protected $environment;


    public function __construct()
    {
        $this->environment = InstanceProvider::getEnvironment();
        $this->entityManager = InstanceProvider::getEntityManager();
    }

    public function findItemtrab($conditions, $page, $pageSize)
    {
        $taskWorkitemRepository = InstanceProvider::getRepository()->getTaskWorkitem();
        $filters = DevOpsUtilities::createQueryFiltered($conditions);
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return $taskWorkitemRepository->findItemtrab($this->environment->getCurrentOrganizationId(), $filters, $limit, $offset);
    }

    public function findItemtrabRelatedToIssuesByTaskId($conditions, $page, $pageSize)
    {
        $taskWorkitemRepository = InstanceProvider::getRepository()->getTaskWorkitem();
        $taskId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'TASK_ID', true);
        $filters = DevOpsUtilities::createQueryFiltered($conditions);
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return $taskWorkitemRepository->findItemtrabRelatedToIssuesByTaskId($this->environment->getCurrentOrganizationId(), $taskId, $filters, $limit, $offset);
    }

    public function save($issueId, $cditemtrab)
    {
        if(empty($cditemtrab)){
            $this->removeAllByIssue($issueId);
        } else {
            $taskWorkitemRepository = InstanceProvider::getRepository()->getTaskWorkitem();
            $taskWorkitemFactory = InstanceProvider::getFactory()->getTaskWorkitem();

            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $issue */
            $issue = InstanceProvider::getRepository()->getTask()->find($issueId);

            foreach ($cditemtrab as $item) {
                $taskWorkitem = $taskWorkitemRepository->findOneBy(array('task' => $issue, 'cditemtrab' => $item));
                if(empty($taskWorkitem)){
                    $taskWorkitem = $taskWorkitemFactory->create($issue, $item);
                    $this->entityManager->persist($taskWorkitem);
                }
            }

            $this->entityManager->flush();

            $taskWorkitems = $taskWorkitemRepository->findBy(array('task' => $issue));

            foreach ($taskWorkitems as $entitie) {
                if(!in_array($entitie->getCditemtrab(), $cditemtrab)){
                    $this->entityManager->remove($entitie);
                }
            }
            $this->entityManager->flush();
        }

    }

    public function removeAllByIssue($issue)
    {
        $taskWorkitemRepository = InstanceProvider::getRepository()->getTaskWorkitem();
        $taskWorkitems = $taskWorkitemRepository->findBy(array('task' => $issue));
        foreach ($taskWorkitems as $entitie) {
            $this->entityManager->remove($entitie);
        }
        $this->entityManager->flush();
    }

}
