<?php
namespace Util\Events\Audit;

use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\Events;
use Doctrine\ORM\Event\LifecycleEventArgs;

class Subscriber implements EventSubscriber {

	protected $environment;

	public function __construct(Environment $environment) {
		$this->environment = $environment;
	}

	public function prePersist(LifecycleEventArgs $args) {
		$entity = $args->getEntity();
		if ($entity instanceof AuditAble) {
			$currentUserId = $this->environment->getCurrentUserId();
			if ($currentUserId) {
				$entity->setCreatedBy($currentUserId);
				$entity->setUpdatedBy($currentUserId);
			}

			$currentTime = $this->environment->getCurrentTime();

			$entity->setCreatedAt($currentTime);
			$entity->setUpdatedAt($currentTime);
		}
	}

	public function preUpdate(LifecycleEventArgs $args) {
		$entity = $args->getEntity();
		if ($entity instanceof AuditAble) {
			$currentUserId = $this->environment->getCurrentUserId();
			if ($currentUserId) {
				$entity->setUpdatedBy($currentUserId);
			}
			$entity->setUpdatedAt($this->environment->getCurrentTime());
		}
	}

	/**
	 * Returns an array of events this subscriber wants to listen to.
	 *
	 * @return array
	 */
	public function getSubscribedEvents() {
		return array(
			Events::prePersist,
			Events::preUpdate,
		);
	}
}